try:
    from twisted.internet import defer, reactor, protocol, endpoints
    from twisted.protocols.basic import LineReceiver
    from txmongo.connection import ConnectionPool
except ImportError:
    print("""This Service Require Following Dependencies:
    twisted
    txmongo

and Databases:
    mongodb
    """)
    exit(1)

import json
import time

from youdao.spider import Spider
__author__ = "hellflame"


class YoudaoProtocol(LineReceiver):
    def __init__(self, factory):
        self.factory = factory

    def connectionMade(self):
        self.factory.connections += 1
        if self.factory.connections > self.factory.MAX_CONNECTIONS:
            self.transport.loseConnection()

    def connectionLost(self, reason=None):
        self.factory.connections -= 1

    def lineReceived(self, line):
        if len(line) > 100:
            # MAX LENGTH 100
            self.transport.loseConnection()

        result = self.factory.get(line)

        def spider_consume(result):
            if result is not None:
                del result['_id']
                del result['key']
                self.transport.write(json.dumps(result) + b'\r\n')
            self.transport.loseConnection()

        def spider_error(err):
            self.transport.loseConnection()

        def done(result):
            if result:
                # print "Mongodb"
                del result['_id']
                del result['key']
                self.transport.write(json.dumps(result) + b'\r\n')
                self.transport.loseConnection()

            else:
                # print "Spider Fetch"
                result = self.factory.spider_fetch(line)
                result.addCallback(spider_consume)
                result.addErrback(spider_error)

        def onError(err):
            self.transport.write('')
            self.transport.loseConnection()

        result.addErrback(onError)
        result.addCallback(done)


class YoudaoFactory(protocol.ServerFactory):
    def __init__(self, uri=None):
        if uri:
            mongodb = ConnectionPool(uri=uri)
        else:
            mongodb = ConnectionPool()
        self.db = mongodb.Youdao.Words
        self.connections = 0
        self.MAX_CONNECTIONS = 1024

    def buildProtocol(self, addr):
        return YoudaoProtocol(self)

    @defer.inlineCallbacks
    def get(self, target):
        result = yield self.db.find_one({
            'key': target
        })
        yield self.db.update({
            'key': target
        }, {
            '$inc': {
                'used': 1
            }
        })
        defer.returnValue(result)

    @defer.inlineCallbacks
    def spider_fetch(self, target):
        status, result = yield Spider().deploy(target)
        if result is not None:
            if not status:
                temp = result
                temp['key'] = target
                temp['insert_time'] = int(time.time())
                yield self.db.insert(temp)
        defer.returnValue(result)


def main(port=3679):
    EndPoint = endpoints.serverFromString(reactor, 'tcp:{}'.format(port))
    EndPoint.listen(YoudaoFactory())
    reactor.run()

if __name__ == '__main__':
    main(5001)




#!/usr/bin/env python2.7
# coding=utf8

import os
import sys
from youdao import Youdao
from sqlsaver import SQLSaver

db_path = SQLSaver().db_path
youdao = Youdao()

__version__ = '4.0.5'
__author__ = "hellflame"

reload(sys)
sys.setdefaultencoding("utf8")

map_target = {
    '--trans': '直接翻译',
    '--web': '网络翻译',
    '--basic': '基本释义',
    '--all': "翻译+基本释义",
    '--help': "显示帮助信息",
    '--clean': "清除数据库",
    "--version": "版本信息",
    '--debug': "调试模式",
    '--comp': "自动补全"
}

short = {
    '--trans': "-t",
    '--web': '-w',
    '--basic': '-b',
    '--all': '-a',
    '--help': '-h',
    '--clean': '-c',
    '--version': '-v',
    '--debug': '-d',
    '--comp': '-cp'
}


def help_menu():
    print "\n有道翻译终端程序\n"
    print "Usage:"
    print "  youdao <word | phrase | sentence> [args...]\t参数后置，查询翻译或解释"
    print "  youdao [args...] <word | phrase | sentence>\t参数前置，查询翻译或解释\n"
    for i in map_target:
        print "  {},{}\t{}".format(i, short[i], map_target[i])

    print "\n输入\033[01;31myoudao + 想要查询的内容\033[00m即可\n"
    print "更多帮助信息 \nhttps://github.com/hellflame/youdao/blob/v{}/README.md\n".format(__version__)


def main():
    argv = sys.argv
    arg_len = len(argv)
    if arg_len == 1:
        help_menu()
    else:
        if arg_len == 2:
            arg = argv[-1]
            if arg in ('-v', '--version'):
                print 'YoudaoDict Version {}'.format(__version__)
            elif arg in ('-c', '--clean'):
                os.remove(db_path)
                print db_path + '  removed'
            elif arg in ('-h', '--help'):
                help_menu()
            elif arg in ('-cp', '--comp'):
                print youdao.complete_code()
            elif arg == '--shard':
                print youdao.shred_auto_complete('')
            else:
                if not arg.startswith("-"):
                    youdao.set_phrase(arg)
                    youdao.executor()
                    result = youdao.basic()
                    web_result = youdao.web()
                    trans = youdao.trans()
                    if result:
                        print result
                    elif not result and youdao.valid and web_result:
                        print web_result
                    elif not result and youdao.valid and trans:
                        print trans
                    else:
                        print " (╯▔皿▔ )╯ \033[01;31m{}\033[00m ㄟ(▔皿▔ ㄟ)".format(arg)
                else:
                    help_menu()
        elif arg_len == 3:
            arg = argv[1:]
            if arg[0].startswith('-'):
                if not arg[0] == '--shard' and not arg[1].startswith("-"):
                    youdao.set_phrase(arg[1])
                    youdao.executor()

                    if arg[0] in ('-d', '--debug'):
                        print youdao.check_raw()
                    elif arg[0] in ('-w', '--web'):
                        print youdao.web()
                    elif arg[0] in ('-t', '--trans'):
                        print youdao.trans()
                    elif arg[0] in ('-b', '--basic'):
                        print youdao.basic()
                    elif arg[0] in ('-a', '--all'):
                        print youdao.basic()
                        print youdao.web()
                        print youdao.trans()
                    else:
                        help_menu()
                elif arg[0] == '--shard':
                    print youdao.shred_auto_complete(arg[1])

            elif arg[1].startswith('-'):
                if not arg[0].startswith('-'):
                    youdao.set_phrase(arg[0])
                    youdao.executor()
                    if arg[1] in ('-d', '--debug'):
                        print youdao.check_raw()
                    elif arg[1] in ('-w', '--web'):
                        print youdao.web()
                    elif arg[1] in ('-t', '--trans'):
                        print youdao.trans()
                    elif arg[1] in ('-b', '--basic'):
                        print youdao.basic()
                    elif arg[1] in ('-a', '--all'):
                        print youdao.basic()
                        print youdao.web()
                        print youdao.trans()
                    else:
                        help_menu()
                else:
                    help_menu()
            else:
                temp = " ".join(arg)
                youdao.set_phrase(temp.strip('-'))
                youdao.executor()
                result = youdao.basic()
                if result:
                    print result
                elif not result and not youdao.is_new and youdao.valid:
                    print youdao.trans()
                else:
                    print " (╯▔皿▔ )╯ \033[01;31m{}\033[00m ㄟ(▔皿▔ ㄟ)".format(youdao.phrase)

        elif arg_len == 4:
            arg = argv[1:]
            temp = " ".join(arg)
            youdao.set_phrase(temp.strip('-'))
            youdao.executor()
            result = youdao.basic()
            if result:
                print result
            elif youdao.result and not youdao.is_new and not result and youdao.valid:
                print youdao.trans()
            else:
                print " (╯▔皿▔ )╯ \033[01;31m{}\033[00m ㄟ(▔皿▔ ㄟ)".format(youdao.phrase)

if __name__ == '__main__':
    main()




import os
import json
import sqlite3
from contextlib import contextmanager
from urllib import quote, unquote
__author__ = "hellflame"


class SQLSaver(object):
    def __init__(self, db_path=''):
        if db_path:
            self.db_path = db_path
        else:
            self.db_path = os.path.expanduser('~/.youdao.sqlite3.db')
        self.TABLE = 'query'

    @contextmanager
    def connection(self):
        try:
            db = sqlite3.connect(self.db_path)
            cursor = db.cursor()
            cursor.execute("CREATE TABLE IF NOT EXISTS `{}` ("
                           "id INTEGER PRIMARY KEY AUTOINCREMENT, "
                           "query varchar(50) NOT NULL UNIQUE,"
                           "raw_json TEXT NOT NULL DEFAULT '')".format(
                            self.TABLE))
            yield cursor
            db.commit()
            db.close()
        except Exception as e:
            raise e

    def query(self, query):
        with self.connection() as cursor:
            cursor.execute("select raw_json from {} WHERE query = '{}' ".format(self.TABLE, query))
            result = cursor.fetchone()
            if result:
                return json.loads(unquote(result[0]))
            return None

    def shred_query(self, shred):
        with self.connection() as cursor:
            if shred:
                cursor.execute("""select query from {} WHERE query like "{}%" limit 10""".format(
                    self.TABLE, shred
                ))
            else:
                cursor.execute("select query from {} limit 10".format(self.TABLE))

            result = cursor.fetchall()
            return result

    def upset(self, query, raw_dict):
        with self.connection() as cursor:
            cursor.execute("select id from {} WHERE query = '{}' ".format(self.TABLE, query))
            result = cursor.fetchone()
            raw_json = quote(json.dumps(raw_dict))
            if result:
                cursor.execute("update {} set raw_json = '{}' WHERE id = {}".format(self.TABLE,
                                                                                    raw_json,
                                                                                    result[0]))
            else:
                cursor.execute("insert into {} (query ,raw_json) VALUES ('{}', '{}')".format(self.TABLE,
                                                                                             query,
                                                                                             raw_json))








# coding=utf8
import spider
import customizer
import sqlsaver
from gevent.monkey import patch_all
from gevent.pool import Pool
from gevent.lock import BoundedSemaphore
from gevent.timeout import Timeout

patch_all()
__author__ = "hellflame"


class Race(object):
    def __init__(self, phrase=''):
        self.phrase = phrase.lower()
        self.pool = Pool()
        self.sem = BoundedSemaphore(3)
        self.result = None
        self.sql_saver = sqlsaver.SQLSaver()

    def race(self, runner):
        with self.sem:
            runner()

    def local_sql_fetch(self):
        with Timeout(1):
            result = self.sql_saver.query(self.phrase)
            if result:
                self.racer_weapon(result, gun='sql')

    def custom_server_fetch(self):
        with Timeout(5):
            result = customizer.Customize(self.phrase).server_fetch()
            if result:
                self.racer_weapon(result, gun='custom')

    def official_server_fetch(self):
        with Timeout(7):
            _, result = spider.Spider().deploy(self.phrase)
            if result:
                self.racer_weapon(result, gun='official')

    def racer_weapon(self, bullet, gun):
        self.result = bullet
        # print gun
        if not gun == 'sql' and 'possibles' not in bullet:
            self.sql_saver.upset(self.phrase, bullet)
        self.pool.kill()

    def launch_race(self):
        try:
            self.pool.map(self.race, [self.custom_server_fetch, self.official_server_fetch, self.local_sql_fetch])
        except:
            pass

if __name__ == '__main__':
    race = Race('world')
    race.launch_race()
    print race.result





# coding=utf8

import sys
import bs4
import urllib
import requests

from contextlib import contextmanager
__author__ = "hellflame"


class Spider(object):
    def __init__(self, lang='eng', timeout=3):
        self.__html_url = "http://dict.youdao.com/w/{}/".format(lang)
        self.__timeout = timeout

    @contextmanager
    def soup(self, target_word):
        url = self.__html_url + urllib.quote(target_word.replace('/', ''))
        try:
            req = requests.get(url, timeout=self.__timeout)
        except requests.Timeout:
            sys.stderr.write('链接 `{}` 请求超时\r\n'.format(url))
            exit(1)
        except requests.ConnectionError:
            sys.stderr.write('链接 `{}` 连接失败\r\n'.format(url))
            exit(1)
        except Exception:
            sys.stderr.write('链接 `{}` 连接时发生未知错误\r\n')
            exit(1)

        if not req.status_code == 200:
            sys.stderr.write('链接 `{}` 非法状态码: {}\r\n'.format(url, req.status_code))
            exit(1)

        yield bs4.BeautifulSoup(req.content, 'html.parser')

    def deploy(self, word):
        with self.soup(word) as soup:
            match = soup.find(class_='keyword')
            if match:
                # pronunciation
                wordbook = soup.find(class_='wordbook-js')
                _pronounce = wordbook.find_all(class_='pronounce')
                pronounces = []
                translate = []
                web_translate = []
                word_phrase = []
                if not _pronounce:
                    _pronounce = wordbook.find_all(class_='phonetic')
                for p in _pronounce:
                    temp = p.get_text().replace(' ', '').replace('\n', '')
                    if not temp:
                        continue
                    pronounces.append(p.get_text().replace(' ', '').replace('\n', ''))

                # translation
                _trans = soup.find(class_='trans-container')
                if _trans and _trans.find('ul'):
                    _normal_trans = _trans.find('ul').find_all('li')
                    if not _normal_trans:
                        _normal_trans = _trans.find('ul').find_all(class_='wordGroup')
                    for _nt in _normal_trans:
                        title = _nt.find(class_='contentTitle')
                        type_ = _nt.find('span')
                        if title and type_:
                            title = title.get_text()
                            type_ = type_.get_text()
                        else:
                            title = _nt.get_text()
                            type_ = ''
                        tmp = (type_ + title).replace('\n', '')
                        if tmp.count(' ') > 4:
                            tmp = tmp.replace("  ", '')
                        translate.append(tmp)

                # web translation
                _web_trans = soup.find(id="tWebTrans")
                if _web_trans:
                    for i in _web_trans.find_all('span', class_=None):
                        temp = i.get_text().replace('\n', '').replace(' ', '')
                        if not temp:
                            continue
                        web_translate.append(temp)

                    # word phrase
                    _word_phrase = _web_trans.find(id='webPhrase')
                    if _word_phrase:
                        for i in _word_phrase.find_all(class_='wordGroup'):
                            title = i.find(class_='contentTitle')
                            if not title:
                                continue
                            title = title.get_text()
                            word_phrase.append({
                                'phrase': title,
                                'explain': i.get_text().replace('\n', '').replace(title, '').replace(' ', '')
                            })

                # print word_phrase
                # print translate
                # print web_translate
                # print pronounces
                return 0, {
                    'pronounces': pronounces,
                    'translate': translate,
                    'web_translate': web_translate
                }
            else:
                # sentence translate may go here, but I won't use youdao. Better use google translate
                similar = soup.find(class_='error-typo')
                if similar:
                    possibles = []
                    similar = similar.find_all(class_='typo-rel')
                    for s in similar:
                        title = s.find(class_='title')
                        content = s.get_text()
                        if title:
                            title = title.get_text().replace(' ', '').replace('\n', '')
                            content = content.replace(title, '').replace(' ', '').replace('\n', '')
                        else:
                            continue
                        possibles.append({
                            'possible': title,
                            'explain': content
                        })
                    return 1, {
                        'possibles': possibles
                    }
                return None, None

if __name__ == '__main__':
    print Spider().deploy('chinese')




# coding=utf8
"""
    fetch result from the backup server, default backup server hellflame.net:3679
    btw: customized server is not a must, but it will speed up a little the query progress
"""

import socket
import json
from contextlib import contextmanager
__author__ = "hellflame"


class Customize(object):
    def __init__(self, target, host='hellflame.net', port=3697):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.settimeout(10)
        self.target = target
        self.address = (host, port)

    @contextmanager
    def connection(self):
        try:
            self.socket.connect(self.address)
            self.socket.send(self.target + b'\r\n')
            # just make this an easy way
            yield self.socket.recv(100000)
            self.socket.close()
        except Exception:
            yield ''

    def server_fetch(self):
        with self.connection() as result:
            if result:
                return json.loads(result)
            return None


if __name__ == '__main__':
    Customize('localhost', 'localhost', 5001).server_fetch()







# coding=utf8
import json
from functools import wraps
from racer import Race
from sqlsaver import SQLSaver
__author__ = "hellflame"


def void_return(fun):
    @wraps(fun)
    def check(self):
        if not self.result or not self.valid:
            return ''
        else:
            return fun(self)
    return check


class Youdao:
    def __init__(self, phrase=''):
        self.phrase = phrase.lower()
        self.result = {}
        self.valid = True
        self.raw = ''
        self.is_new = False

    def set_phrase(self, phrase):
        self.phrase = phrase.lower()

    def valid_check(self):
        if not self.result:
            self.is_new = True
            self.valid = False
            return False

        if 'errorCode' not in self.result:
            self.is_new = True

        if not self.is_new:
            if 'translation' not in self.result or \
                    (len(self.result['translation']) == 1 and self.result['translation'][0] == self.result['query']):
                self.valid = False

    def executor(self):
        race = Race(self.phrase)
        race.launch_race()
        self.result = race.result
        self.valid_check()
        return self.result

    def check_raw(self):
        return json.dumps(self.result, indent=2)

    @void_return
    def web(self):
        temp = ""
        if not self.is_new:
            if "web" not in self.result:
                return temp
            temp += "网络释义 \033[01;34m>>>\033[00m\n"
            for i in self.result['web']:
                temp += '\t{}\n\t'.format(i['key'])
                for j in i['value']:
                    temp += '  ' + j + ','
                temp += '\n'
        else:
            if 'web_translate' in self.result and self.result['web_translate']:
                temp += "网络释义 \033[01;34m>>>\033[00m\n"
                for i in self.result['web_translate']:
                    temp += '\t{}\n'.format(i)

        return temp.strip()

    @void_return
    def trans(self):
        temp = ""
        if not self.is_new:
            temp += '翻译     \033[01;33m>>>\033[00m\n'
            for i in self.result['translation']:
                temp += '\t{}\n'.format(i)
        else:
            possibles = self.result.get("possibles", [])

            if possibles:
                temp += "相关词语     \033[01;33m>>>\033[00m\n"
                for i in possibles:
                    temp += "\t{}\n\t{}\n\n".format(i['possible'], i['explain'])
            else:
                temp = ""

        return temp

    @void_return
    def basic(self):
        temp = ""
        if not self.is_new:
            data = self.result.get('basic', '')
            if not data:
                return temp
            temp += "基本释义 \033[01;32m>>>\033[00m\n"
            phonetic = data.get('phonetic', '')
            us_phonetic = data.get('us-phonetic', '')
            uk_phonetic = data.get('uk-phonetic', '')
            base = data.get('explains', '')
            if phonetic:
                temp += "\t[{}]\n".format(phonetic)
            if us_phonetic:
                temp += "\tus. [{}]\n".format(us_phonetic)
            if uk_phonetic:
                temp += "\tuk. [{}]\n".format(uk_phonetic)
            if base:
                for i in base:
                    temp += "\t{}\n".format(i)
        else:
            trans = self.result.get('translate', [])
            if trans:
                temp += "基本释义 \033[01;32m>>>\033[00m\n"
                pronounce = self.result.get("pronounces", [])
                for i in pronounce:
                    temp += "\t{}\n".format(i)
                for i in trans:
                    temp += "\t{}\n".format(i)
            else:
                temp = ""

        return temp

    @staticmethod
    def shred_auto_complete(shred):
        shreds = SQLSaver().shred_query(shred)
        temp = " ".join((x[0] for x in shreds if len(shreds) > 1 and not x[0] == shred or x[0].startswith(shred)))
        return temp

    @staticmethod
    def complete_code():
        return """###-begin-youdao-completion-###
# simple youdaoDict word auto completion script
# Installation: youdao -cp >> ~/.bashrc  (or ~/.zshrc)
# or youdao -cp >> ~/.bash_profile (.etc)
#
_youdao_parser_options()
{
  local curr_arg;
  curr_arg=${COMP_WORDS[COMP_CWORD]}
  COMPREPLY=( $(compgen -W "$(youdao --shard $curr_arg)" $curr_arg ) );
}
complete -F _youdao_parser_options youdao
###-end-youdao-completion-###"""


if __name__ == '__main__':
    youdao = Youdao()
    print (youdao.shred_auto_complete('f'))


